from errno import EEXIST
from os import makedirs, remove
from os.path import join, dirname

from oslo_config import cfg

from magellan.common.exceptions import MagellanException


CONF = cfg.CONF

DEFAULT_DATA_DIR = 'data'
IP_LIST_DIR = 'ip_list'


def get_data_dir():
    data_dir = CONF.get('data_dir')
    if not data_dir:
        raise MagellanException('Missing data_dir configuration')
    return data_dir


def write_data_file(filename, data):
    filename = join(get_data_dir(), filename)
    try:
        makedirs(dirname(filename))
    except OSError as e:
        if e.errno != EEXIST:
            raise
    with open(filename, 'w') as f:
        f.write(data)


def read_data_file(filename):
    filename = join(get_data_dir(), filename)
    try:
        with open(filename, 'r') as f:
            return f.read()
    except FileNotFoundError:
        return ''


def write_ip_list(job_id, ip_list):
    write_data_file(join(IP_LIST_DIR, str(job_id)), ip_list)


def read_ip_list(job_id):
    return read_data_file(join(IP_LIST_DIR, str(job_id)))


def get_ip_list_file(job_id):
    return join(get_data_dir(), IP_LIST_DIR, str(job_id))


def remove_ip_list(job_id):
    try:
        remove(get_ip_list_file(job_id))
    except FileNotFoundError:
        pass
