import sqlalchemy as sa

from magellan.db.sqlalchemy.types import GUID, JSONEncodedDict


def upgrade(migrate_engine):
    meta = sa.MetaData()
    meta.bind = migrate_engine
    discovery_devices = sa.Table(
        "devices",
        meta,
        sa.Column("id", GUID, nullable=False, primary_key=True),
        sa.Column(
            "execution_id",
            GUID,
            sa.ForeignKey("executions.id", ondelete="CASCADE"),
            nullable=False,
        ),
        sa.Column("address", sa.String(14), nullable=False),
        sa.Column("details", JSONEncodedDict(1023), nullable=False),
        sa.Column("created_at", sa.DateTime),
        sa.Column("updated_at", sa.DateTime),
        sa.Column("deleted_at", sa.DateTime),
        mysql_engine="InnoDB",
        mysql_charset="utf8",
    )
    config = sa.Column("config", JSONEncodedDict(1023), nullable=True)
    discovery_devices.create_column(config)


def downgrade(migrate_engine):
    meta = sa.MetaData()
    meta.bind = migrate_engine
    discovery_devices = sa.Table(
        "devices",
        meta,
        sa.Column("id", GUID, nullable=False, primary_key=True),
        sa.Column(
            "execution_id",
            GUID,
            sa.ForeignKey("executions.id", ondelete="CASCADE"),
            nullable=False,
        ),
        sa.Column("address", sa.String(14), nullable=False),
        sa.Column("details", JSONEncodedDict(1023), nullable=False),
        sa.Column("created_at", sa.DateTime),
        sa.Column("updated_at", sa.DateTime),
        sa.Column("deleted_at", sa.DateTime),
        sa.Column("config", JSONEncodedDict(1023), nullable=True),
        mysql_engine="InnoDB",
        mysql_charset="utf8",
    )
    discovery_devices.drop_column("config")
