import uuid
import sqlalchemy as sa

from magellan.db.sqlalchemy.types import GUID, JSONEncodedDict
from magellan.common.consts import ExecutionStatus, ScheduleStatus, ScanMethod


def upgrade(migrate_engine):
    meta = sa.MetaData()
    meta.bind = migrate_engine

    discovery_jobs = sa.Table(
        'jobs', meta,
        sa.Column(
            'id', GUID, primary_key=True, default=uuid.uuid4, nullable=False
        ),
        sa.Column('name', sa.String(255), unique=True, nullable=False),
        sa.Column('description', sa.String(1023)),
        sa.Column('method', sa.Enum(ScanMethod), nullable=False),
        sa.Column('options', JSONEncodedDict(1023), nullable=False),
        sa.Column('timeout', sa.Integer, default=3600, nullable=False),
        sa.Column('schedule', sa.String(255)),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('deleted_at', sa.DateTime),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    discovery_jobs.create_column(sa.Column('username', sa.String(255)))

    discovery_executions = sa.Table(
        'executions', meta,
        sa.Column('id', GUID,
                  primary_key=True, nullable=False),
        sa.Column('job_id', GUID,
                  sa.ForeignKey('jobs.id', ondelete='SET NULL')),
        sa.Column('name', sa.String(255),
                  nullable=False),
        sa.Column('description', sa.String(1023),
                  nullable=True),
        sa.Column('status', sa.Enum(ExecutionStatus),
                  nullable=False),
        sa.Column('complete_time', sa.DateTime),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('deleted_at', sa.DateTime),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    discovery_executions.create_column(sa.Column(
        'schedule_status',
        sa.Enum(ScheduleStatus),
        default=ScheduleStatus.unscheduled,
    ))
    discovery_executions.create_column(sa.Column('username', sa.String(255)))


def downgrade(migrate_engine):
    meta = sa.MetaData()
    meta.bind = migrate_engine

    discovery_jobs = sa.Table(
        'jobs', meta,
        sa.Column(
            'id', GUID, primary_key=True, default=uuid.uuid4, nullable=False
        ),
        sa.Column('name', sa.String(255), unique=True, nullable=False),
        sa.Column('description', sa.String(1023)),
        sa.Column('method', sa.Enum(ScanMethod), nullable=False),
        sa.Column('options', JSONEncodedDict(1023), nullable=False),
        sa.Column('timeout', sa.Integer, default=3600, nullable=False),
        sa.Column('schedule', sa.String(255)),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('deleted_at', sa.DateTime),
        sa.Column('username', sa.String(255)),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    discovery_jobs.drop_column('username')

    discovery_executions = sa.Table(
        'executions', meta,
        sa.Column('id', GUID,
                  primary_key=True, nullable=False),
        sa.Column('job_id', GUID,
                  sa.ForeignKey('jobs.id', ondelete='SET NULL')),
        sa.Column('name', sa.String(255),
                  nullable=False),
        sa.Column('description', sa.String(1023),
                  nullable=True),
        sa.Column('status', sa.Enum(ExecutionStatus),
                  nullable=False),
        sa.Column('complete_time', sa.DateTime),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('deleted_at', sa.DateTime),
        sa.Column(
            'schedule_status',
            sa.Enum(ScheduleStatus),
            default=ScheduleStatus.unscheduled,
        ),
        sa.Column('username', sa.String(255)),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    discovery_executions.drop_column('schedule_status')
    discovery_executions.drop_column('username')
