from oslo_concurrency import processutils
from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import uuidutils
from magellan.db.api import get_engine, db_sync
from magellan.engine import base as service_base
from magellan.engine import engine as default_engine
from magellan.engine import scheduler
from magellan.rpc import server as rpc


LOG = logging.getLogger(__name__)
CONF = cfg.CONF


class EngineServer(service_base.MagellanService):
    """Engine server.
    This class manages engine life-cycle and sets up an RPC server to process
    engine specific calls.
    """

    def __init__(self, engine):
        super(EngineServer, self).__init__()
        self.engine = engine
        self.workers = (
            CONF.engine.engine_workers or processutils.get_worker_count()
        )
        self.rpc_server = rpc.RpcServer(self)
        self.scheduler = scheduler.JobScheduler()

    def start(self):
        self.engine_id = uuidutils.generate_uuid()
        if CONF.db_sync:
            db_sync(get_engine())
            LOG.info("Magellan DB synced")
        super(EngineServer, self).start()
        self.rpc_server.start()
        self.scheduler.start()
        LOG.info("Magellan engine server started")
        self._notify_started()

    def stop(self, graceful=False):
        self.rpc_server.stop()
        super(EngineServer, self).stop(graceful)
        LOG.info("Magellan engine server stopped")


def get_oslo_service():
    return EngineServer(default_engine.DefaultEngine())
