import mock
import pytest

from magellan.engine.lmd import socket, LmdConnection


@pytest.fixture
def mock_socket(monkeypatch):
    socket_obj_mock = mock.MagicMock()
    socket_fun_mock = mock.Mock(return_value=socket_obj_mock)
    monkeypatch.setattr(socket, "socket", socket_fun_mock)
    return socket_obj_mock


def test_connect(mock_socket):
    conn = LmdConnection()
    conn.connect()
    assert conn._sock is mock_socket
    mock_socket.connect.assert_has_calls(
        [mock.call("/opt/monitor/var/rw/live")]
    )


def test_close(mock_socket):
    conn = LmdConnection()
    conn.connect()
    conn.close()
    assert conn._sock is None
    mock_socket.close.assert_has_calls([mock.call()])


def test_query(mock_socket):
    mock_socket.recv.side_effect = [b'[["foo"],["bar"]]', b""]
    with LmdConnection() as conn:
        result = conn.query("GET hosts\nColumns: name\n")
    mock_socket.sendall.assert_has_calls(
        [mock.call(b"GET hosts\nColumns: name\n")]
    )
    assert result == '[["foo"],["bar"]]'


def test_json(mock_socket):
    mock_socket.recv.side_effect = [b'[["foo"],["bar"]]', b""]
    with LmdConnection() as conn:
        result = conn.json("GET hosts\nColumns: name\n")
    assert result == [["foo"], ["bar"]]
