import wsme
import wsmeext.pecan
from oslo_config import cfg
from oslo_log import log as logging
from http.client import responses

LOG = logging.getLogger(__name__)


def expose(*args, **kwargs):
    """
    Wrapper around `wsmeext.pecan.wsexpose` that lets us inject some default
    configuration options into its parameters.
    """
    kwargs.setdefault("rest_content_types", cfg.CONF.api.content_types)
    return wsmeext.pecan.wsexpose(*args, **kwargs)


def monkeypatch_wsme():
    """
    WSME does not provide any method for customizing the data returned when
    an error occurs. This will monkey patch WSME to return error messages on
    our preferred format.
    """

    def monkeypatch_method(cls):
        def decorator(func):
            setattr(cls, func.__name__, func)
            return func
        return decorator

    @monkeypatch_method(wsme.api)
    def format_exception(excinfo, debug=False):
        error = excinfo[1]
        code = getattr(error, 'code', 500)
        # Log the error messages from both magellan serve and client
        forward_msg = (getattr(error, 'message', '')
                       or getattr(error, 'faultstring', '')
                       or getattr(error, 'value', ''))
        if forward_msg:
            LOG.info(forward_msg)
        else:
            # Log exceptions from internal messages from both serve and api
            LOG.error(error, exc_info=True)

        return {
            'name': responses.get(code, 'Unknown Error'),
            'description': forward_msg,
            'errors': []
        }

    @monkeypatch_method(wsmeext.pecan.JSonRenderer)
    def render(_, template_path, namespace):
        if 'result' in namespace:
            return wsme.rest.json.encode_result(
                namespace['result'],
                namespace['datatype']
            )
        else:
            return wsme.rest.json.encode_error(None, namespace)
